part of backend;

String _modelTemp(String name, List<String> attrs) {
  final modelName = name.toCamelcase();
  final attr = StringBuffer();
  final fromMap = StringBuffer();
  final toMap = StringBuffer();
  final constructor = StringBuffer();
  final copyWithAttr = StringBuffer();
  final copyWithBody = StringBuffer();
  for (final value in attrs) {
    attr.writeln('String? $value;');
    fromMap.writeln('model.$value = data["$value"];');
    toMap.writeln("if ($value != null) '$value': $value,");
    constructor.writeln('this.$value,');
    copyWithAttr.writeln('String? $value,');
    copyWithBody.writeln('this.$value = $value ?? this.$value;');
  }
  return '''
import 'dart:convert';

import 'package:cloud_firestore/cloud_firestore.dart';
import '../controller_manager.dart';

class $modelName extends Model {
  ${attr.toString()}

  static const collectionName = '$modelName';
  static CollectionReference get ref => FirebaseFirestore.instance.collection(collectionName); 


  $modelName(
  {${constructor.toString()}}
  );

  factory $modelName.fromSnapshot(DocumentSnapshot snapshot) {
    final data = snapshot.data() as Map? ?? {};
    final model = $modelName();
    model.id = snapshot.id;
    ${fromMap.toString()}
    return model;
  }
  
  @override
  Map<String, dynamic> toMap() => {
      ${toMap.toString()}
  };

  String toJson() => json.encode(toMap());
  
  
  $modelName copyWith({
    ${copyWithAttr.toString()}
  }) {
    ${copyWithBody.toString()}
    return this;
  }
}
''';
}

const _controllerManagerTemp = '''
import 'package:bot_toast/bot_toast.dart';
import 'package:cloud_firestore/cloud_firestore.dart';

abstract class Model {
  String? id;

  Map<String, dynamic> toMap();
}

void _onError(Exception e) =>
    BotToast.showSimpleNotification(title: e.toString());


Future tryRun(Function action, [Function(Exception)? onError]) async {
  try {
    final result = await action();
    return result ?? true;
  } catch (e) {
    onError?.call(e as Exception);
    return null;
  }
}

class Controller<T extends Model> {
  final CollectionReference ref;

  Controller(String collectionName)
      : ref = FirebaseFirestore.instance.collection(collectionName);

  Future<DocumentReference?> insert(T model) async {
    final result = await tryRun(() => ref.add(model.toMap()), _onError);
    return result;
  }

  Future<DocumentSnapshot?> find(String id) async {
    final result = await tryRun(() => ref.doc(id).get(), _onError);
    return result;
  }

  Future<QuerySnapshot?> get(
      {DocumentSnapshot? afterDoc,
      dynamic after,
      int? limit,
      String? orderBy,
      bool desc = false}) async {
    late Query query = ref;
    if (limit != null) {
      query = query.limit(limit);
    }
    if (orderBy != null) {
      query = query.orderBy(orderBy, descending: desc);
    }
    if (afterDoc != null) {
      query = query.startAfterDocument(afterDoc);
    }
    if (after != null) {
      query = query.startAfter([after]);
    }

    final result = await tryRun(() => query.get(), _onError);
    return result;
  }

  Future<bool?> update(T model) async {
    final result =
        await tryRun(() => ref.doc(model.id).update(model.toMap()), _onError);
    return result;
  }

  Future<bool?> remove(T model) async {
    final result = await tryRun(() => ref.doc(model.id).delete(), _onError);
    return result;
  }

  Future<bool?> drop() async {
    var dropResult = true;
    final result = await tryRun(() => get(), _onError);
    if (result == null) return result;
    for (final doc in (result as QuerySnapshot).docs) {
      try {
        doc.reference.delete();
      } catch (_) {
        dropResult = false;
      }
    }
    return dropResult ? true : null;
  }

  Future query(Query query) async {
    final result = await tryRun(query.get, _onError);
    return result;
  }
}
''';

String _controllerTemp(String name) => '''
import '../controller_manager.dart';
import '../model/models.dart';

class ${name}Controller{
  
}
''';

const _storageManagerTemp = '''
import 'dart:io';

import 'package:bot_toast/bot_toast.dart';
import 'package:firebase_storage/firebase_storage.dart';

void _onError(Exception e) =>
    BotToast.showSimpleNotification(title: e.toString());


Future tryRun(Function action, [Function(Exception)? onError]) async {
  try {
    final result = await action();
    return result ?? true;
  } catch (e) {
    onError?.call(e as Exception);
    return null;
  }
}

class StorageManager {
  final String on;
  late final _storage = FirebaseStorage.instance.ref(on);

  StorageManager(this.on);

  Future upload(File file, [String? fileName]) async {
    final name = fileName ??= file.path.split('/').last;
    final result =
        await tryRun(() => _storage.child(name).putFile(file), _onError);
    if (result == null) return null;
    final downloadLink =
        await tryRun(() => _storage.child(name).getDownloadURL(), _onError);
    return downloadLink;
  }

  Future delete(String url) async {
    final result = await tryRun(
        () => FirebaseStorage.instance.refFromURL(url).delete(), _onError);
    return result;
  }
}

''';
